
import multiprocessing as mp


class Manager(object):

    def __init__(self, target, threads=1, **kwargs):
        self.workers = []
        self.target = target
        self.threads = int(threads)

    def start(self, **kwargs):
        for n in range(self.threads):
            p = mp.Process(target=self.target(**kwargs).start)
            p.start()
            self.workers.append(p)

        return self.workers

    def stop(self):

        if hasattr(self, 'teardown'):
            self.teardown()

        for p in self.workers:
            p.terminate()



from cifsdk.zmq.msg import Msg
import zmq
from cifsdk.zmq.socket import Context
from cifsdk.zmq.utils import unpack
import msgpack


def test_msgs():

    def _send_multipart(m):
        assert msgpack.unpackb(m[0]) == Msg.INDICATORS_SEARCH
        assert unpack(m[1]) == []

    m = Msg(mtype=Msg.INDICATORS_SEARCH, data=[])

    ctx = Context()
    s = ctx.socket(zmq.REQ)
    s.send_multipart = _send_multipart

    s.send_msg(m)


def test_msgs_recv():

    def _recv_multipart():
        return Msg(id=msgpack.packb(1234),
                   mtype=Msg.INDICATORS_SEARCH, data=[]).to_frame()

    ctx = Context()
    s = ctx.socket(zmq.REQ)
    s.recv_multipart = _recv_multipart

    m = s.recv_msg()

    assert msgpack.unpackb(m.id) == 1234
    assert m.mtype == 'indicators_search'
    assert m.data == []



import json
import logging
import os
from msgpack import packb, unpackb
from msgpack.exceptions import ExtraData
from cifsdk.zmq.utils import pack, unpack, mtype_to_int, MAP, DecimalDecoder

try:
    import zmq

except ImportError:
    print("Requires pyzmq")
    raise SystemExit

TRACE = os.getenv('CIFSDK_MSG_TRACE', False)

logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)

if TRACE == '1':
    logger.setLevel(logging.DEBUG)


class Msg(object):

    INDICATORS_CREATE = 1
    INDICATORS_SEARCH = 2
    INDICATORS_DELETE = 3

    def __init__(self, **kwargs):
        for e in ['id', 'client_id', 'mtype', 'data']:
            if isinstance(kwargs.get(e), bytes):
                try:
                    kwargs[e] = kwargs[e].decode('utf-8')
                except:
                    pass
            setattr(self, e, kwargs.get(e))

        if isinstance(self.data, bytes):
            self.data = unpack(self.data)

    def __repr__(self):
        return json.dumps(self.__dict__(), cls=DecimalDecoder)

    def __dict__(self):
        return {
            'id': self.id,
            'mtype': self.mtype,
            'data': self.data,
        }

    def to_frame(self):
        m = []

        # check to see if we have a frame / client id
        for e in ['id', 'client_id']:
            if getattr(self, e):
                m.append(getattr(self, e))

        # insert a null
        if len(m) > 0:
            m.append(''.encode('utf-8'))

        # msg type
        if isinstance(self.mtype, bytes):
            self.mtype = mtype_to_int(self.mtype.decode('utf-8'))

        elif isinstance(self.mtype, str):
            self.mtype = mtype_to_int(self.mtype)

        m.append(packb(self.mtype))

        # pack it up
        if isinstance(self.data, dict):
            self.data = [self.data]

        m.append(pack(self.data))

        # encode all the strings
        for idx, v in enumerate(m):
            if isinstance(v, str):
                m[idx] = v.encode('utf-8')

        return m

    @staticmethod
    def from_frame(m):
        # zmq has diff frame types depending on
        if len(m) == 5:
            id, client_id, _, mtype, data = m
            return Msg(id=id, client_id=client_id, mtype=MAP[unpackb(mtype)],
                       data=unpack(data))

        if len(m) == 4:
            id, _, mtype, data = m
            return Msg(id=id, mtype=MAP[unpackb(mtype)], data=unpack(data))

        if len(m) == 3:
            id, mtype, data = m

            try:
                mtype = unpackb(mtype)

            except ExtraData:
                pass

            if len(id) > 10:
                return Msg(mtype=MAP[mtype], data=unpack(data))

            return Msg(id=id, mtype=MAP[mtype], data=unpack(data))

        mtype, data = m
        return Msg(mtype=mtype, data=unpack(data))



import logging
from cifsdk.client.zeromq.base import Base
from cifsdk.client import Client

from cifsdk.zmq.msg import Msg
from csirtg_indicator import Indicator
from .constants import TRACE, FIREBALL_SIZE

logger = logging.getLogger(__name__)

logger.setLevel(logging.ERROR)


class ZMQ(Client, Base):

    def _should_fireball(self, data):
        if self.fireball or len(data) > 25:
            return True

    def indicators_search(self, filters, decode=True):
        return self._send(Msg.INDICATORS_SEARCH, filters,
                          decode=decode)

    def _prepare_data(self, data):
        if isinstance(data, dict):
            data = self._kv_to_indicator(data)

        if type(data) == dict:
            data = [data]

        if isinstance(data, list) and isinstance(data[0], Indicator):
            data = [i.__dict__() for i in data]

        if isinstance(data, Indicator):
            data = [data.__dict__()]

        return data

    def indicators_create(self, data=[], nowait=False, fireball=None,
                          f_size=FIREBALL_SIZE):

        if isinstance(data, list) and len(data) == 0:
            return

        data = self._prepare_data(data)

        if fireball is False:
            return self._send(Msg.INDICATORS_CREATE, data, nowait=nowait)

        if fireball or self._should_fireball(data):
            return self._send_fireball(data, f_size)

        return self._send(Msg.INDICATORS_CREATE, data, nowait=nowait)

    def indicators_delete(self, data):
        if isinstance(data, dict):
            data = self._kv_to_indicator(data)

        if isinstance(data, Indicator):
            data = str(data)

        return self._send(Msg.INDICATORS_DELETE, data)
